﻿using Microsoft.Extensions.Configuration;
using NpgsqlTypes;
using Serilog;
using Serilog.Sinks.PostgreSQL;
using System;
using System.Collections.Generic;
using System.IO;

namespace Correo.Extensions
{
    public static class LoggingExtensions
    {
        internal static LoggerConfiguration WriteToConfiguredDestinations(this LoggerConfiguration sinkConfiguration, IConfiguration configuration)
        {
            var workspace = configuration.GetValue<string>("Workspace");
            var fileEnabled = configuration.GetValue<bool>("Logs:File:Enabled");
            var filePath = configuration.GetValue<string>("Logs:File:Path");
            var seqEnabled = configuration.GetValue<bool>("Logs:Seq:Enabled");
            var seqUrl = configuration.GetValue<string>("Logs:Seq:Url");
            var seqApiKey = configuration.GetValue<string>("Logs:Seq:ApiKey");
            var postgresEnabled = configuration.GetValue<bool>("Logs:Postgres:Enabled");
            var postgresConnection = configuration.GetValue<string>("Logs:Postgres:Connection");

            if (fileEnabled && string.IsNullOrWhiteSpace(filePath))
                throw new Exception("If file logging is enabled, the log file path must be configured.");
            if (seqEnabled && string.IsNullOrWhiteSpace(seqUrl))
                throw new Exception("If Seq logging is enabled, the Seq URL must be configured.");
            if (postgresEnabled && string.IsNullOrWhiteSpace(postgresConnection))
                throw new Exception("If Postgres logging is enabled, the Postgres connection must be configured.");

            if (filePath.NotEmpty() && workspace.NotEmpty())
                filePath = Path.Combine(workspace, filePath);

            if (fileEnabled)
                sinkConfiguration.WriteTo.File(filePath, rollingInterval: RollingInterval.Day);

            if (seqEnabled)
                sinkConfiguration.WriteTo.Seq(seqUrl, apiKey: seqApiKey.Nullify());

            if (postgresEnabled)
            {
                var columnWriters = new Dictionary<string, ColumnWriterBase>
                {
                    {"message", new RenderedMessageColumnWriter(NpgsqlDbType.Text) },
                    {"level", new LevelColumnWriter(true, NpgsqlDbType.Varchar) },
                    {"raise_date", new TimestampColumnWriter(NpgsqlDbType.Timestamp) },
                    {"exception", new ExceptionColumnWriter(NpgsqlDbType.Text) },
                    {"event", new LogEventSerializedColumnWriter(NpgsqlDbType.Jsonb) },
                    {"properties", new PropertiesColumnWriter(NpgsqlDbType.Jsonb) }
                };

                sinkConfiguration.WriteTo.PostgreSQL(postgresConnection, "correo_logs", columnWriters, needAutoCreateTable: true);
            }

            return sinkConfiguration;
        }
    }
}
